package app;

import javax.swing.JPanel;
import java.awt.*;
import java.awt.print.*;

/**
 * Definiert eine Pane, die sowohl am Bildschirm angezeigt als auch gedruckt
 * werden kann.<br>
 * Bug: noch kein Mehrseiten-Druck.<br>
 * <i>(C)opyright 2002 L.H.Hahn</i>
 * @author Dr. Lars H. Hahn
 * @version 3.0
 */

abstract public class AbstractPrintablePane
  extends JPanel
  implements Printable {

  private Font fontPlain;
  private Font fontBold;
  private final String ueberschrift;

  /**
   * Einziger Konstruktor.
   * @param ueberschrift Die berschrift wird <b>fett</b> oben links ausgegeben.
   */
  public AbstractPrintablePane(String ueberschrift) {
    this.ueberschrift = ueberschrift;
    setFont("Helvetica", 12);
  }

  /**
   * Setzt den Zeichensatz der Tabelle.
   * @param name Name des Zeichensatzes
   * @param punkt Gre des Zeichensatzes (in Punkt)
   */
  public void setFont(String name, int punkt) {
    fontPlain = new Font(name, Font.PLAIN, punkt);
    fontBold = new Font(name, Font.BOLD, punkt);
  }

  /**
   * Ermglicht der abgeleiteten Klasse unabhgig vom Bildschirm oder vom
   * Drucker zu zeichnen. Die Parameter geben die Rnder der Zeichenflche
   * an. Gedruckt/Angezeigt wird diese Pane mit anzeigenOderDrucken() der Klasse
   * Applikation.
   * @param fensterX X-Koordinate des Ursprungs der Zeichenflche
   * @param fensterY Y-Koordinate des Ursprungs der Zeichenflche
   * @param fensterBreite nutzbare Breite der Zeichenflche
   * @param fensterHoehe nutzbare Hhe der Zeichenflche
   * @see Applikation
   */
  abstract public void zeichnen(Graphics g, int fensterX, int fensterY, int fensterBreite, int fensterHoehe);

  public void paintComponent(Graphics g) {
    setBackground(Color.white);
    g.setFont(fontBold);
    int ueberschriftHoehe = g.getFontMetrics().getHeight();
    super.paintComponent(g); //paint background
    g.drawString(ueberschrift, 10, ueberschriftHoehe);
    g.setFont(fontPlain);
    Dimension dimPane = getSize();
    zeichnen(g,
             10,
             ueberschriftHoehe+5,
             (int)dimPane.getWidth()-20,
             (int)dimPane.getHeight()-10-ueberschriftHoehe);
  }

  public int print(Graphics g, PageFormat pageFormat, int pageIndex) {
    g.setFont(fontBold);
    int ueberschriftHoehe = 2 * g.getFontMetrics().getHeight();
    if(pageIndex != 0)
      return NO_SUCH_PAGE;
    int x = (int)(pageFormat.getImageableX() + 0.5D);
    int y = (int)(pageFormat.getImageableY() + 0.5D) + ueberschriftHoehe;
    int dx = (int)(pageFormat.getImageableWidth() + 0.5D) - 1;
    int dy = (int)(pageFormat.getImageableHeight() + 0.5D) - ueberschriftHoehe - 1;
    g.drawString(ueberschrift, x, y);
    g.setFont(fontPlain);
    zeichnen(g, x, y, dx, dy);
    return PAGE_EXISTS;
  }

}