package app;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.ArrayList;

/**
 * Dialog zum ndern der Optionen der Applikation.<br>
 * <i>(C)opyright 2002 L.H.Hahn</i>
 * @author Dr. Lars H. Hahn
 * @version 3.0
 */

public class DlgOptionen
  extends JDialog {

  private final boolean DEBUG = false;

  private ArrayList checkBoxes = new ArrayList();
  private ArrayList checkBoxCommands = new ArrayList();
  private ArrayList textFields = new ArrayList();
  private ArrayList textFieldCommands = new ArrayList();
  private ArrayList comboBoxes = new ArrayList();
  private ArrayList comboBoxCommands = new ArrayList();
  private final JPanel masterPane = new JPanel();
  private final JPanel buttonPane = new JPanel();
  private final JButton buttonOK = new JButton("OK");

  public DlgOptionen(JFrame of) {
    super(of, "Optionen", true); // = modal
    masterPane.setLayout(new GridLayout(0,2));
    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setVisible(false);
      }
    });
    buttonPane.add(buttonOK);
    getRootPane().setDefaultButton(buttonOK);
    Container contentPane = this.getContentPane();
    contentPane.add(masterPane, BorderLayout.CENTER);
    contentPane.add(buttonPane, BorderLayout.SOUTH);
  }

  /**
   * Zeigt den Dialog auf dem Bildschirm an, wenn er noch nicht sichtbar ist.
   */
  public void showDialog() {
    if(!isShowing()) {
      setLocationRelativeTo(this.getOwner());
      pack();
      setVisible(true);
    }
  }

  /**
   * Fgt dem Optionsdialog eine Check-Box hinzu.
   * @param actionCommand Schlssel mit dem der Zustand der Check-Box mit Hilfe
   *                      von getWert() abgefragt werden kann
   * @param text          Text der Check-Box
   * @param wahl          initialer Zustand der Check-Box
   */
  public void addCheckBox(String actionCommand, String text, boolean wahl) {
    JCheckBox checkBox = new JCheckBox("", wahl);
    JLabel label = new JLabel(text);
    masterPane.add(label);
    masterPane.add(checkBox);
    checkBoxes.add(checkBox);
    checkBoxCommands.add(actionCommand);
    if(DEBUG)
      System.out.println("CheckBox: "+actionCommand+" mit Wert: "+wahl);
  }

  /**
   * Fgt dem Optionsdialog ein Textfeld hinzu.
   * @param actionCommand Schlssel mit dem der Zustand des Textfeldes mit Hilfe
   *                      von getWert() abgefragt werden kann
   * @param text          erlutender Text vor dem Textfeld
   * @param wert          initialer Inhalt des Textfeldes
   */
  public void addTextField(String actionCommand, String text, String wert) {
    JLabel label = new JLabel(text);
    JTextField textField = new JTextField(wert);
    textFields.add(textField);
    masterPane.add(label);
    masterPane.add(textField);
    textFieldCommands.add(actionCommand);
    if(DEBUG)
      System.out.println("TextField: "+actionCommand+" mit Wert: "+wert);
  }

  /**
   * Fgt dem Optionsdialog eine Combo-Box hinzu.
   * @param actionCommand Schlssel mit dem der Zustand der Check-Box mit Hilfe
   *                      von getWert() abgefragt werden kann
   * @param text          erlutender Text vor der Combo-Box
   * @param werte         Alternativen der Combo-Box
   */
  public void addComboBox(String actionCommand, String text, Object[] werte, Object gewaehlterWert) {
    JLabel label = new JLabel(text);
    JComboBox comboBox = new JComboBox(werte);
    comboBox.setSelectedItem(gewaehlterWert);
    comboBoxes.add(comboBox);
    masterPane.add(label);
    masterPane.add(comboBox);
    comboBoxCommands.add(actionCommand);
    if(DEBUG)
      System.out.println("ComboBox: "+actionCommand+" mit Wert: "+gewaehlterWert);
  }

  /**
   * Liefert der Wert der Check-Box / des Textfeldes / der Combo-Box.
   * @param actionCommand Schlssel der beim Hinzufgen definiert wurde
   * @return Boolean (Check-Box), String (Textfeld) oder Object (Combo-Box)
   */
  public Object getWert(String actionCommand) {
    for(int i=0; i<textFieldCommands.size(); i++) {
      if(actionCommand.equals(textFieldCommands.get(i))) {
        JTextField textField = (JTextField)textFields.get(i);
        if(DEBUG)
          System.out.println("getWert("+actionCommand+") = "+textField.getText());
        return textField.getText();
      }
    }
    for(int i=0; i<checkBoxCommands.size(); i++) {
      if(actionCommand.equals(checkBoxCommands.get(i))) {
        JCheckBox checkBox = (JCheckBox)checkBoxes.get(i);
        if(DEBUG)
          System.out.println("getWert("+actionCommand+") = "+checkBox.isSelected());
        return new Boolean(checkBox.isSelected());
      }
    }
    for(int i=0; i<comboBoxCommands.size(); i++) {
      if(actionCommand.equals(comboBoxCommands.get(i))) {
        JComboBox comboBox = (JComboBox)comboBoxes.get(i);
        if(DEBUG)
          System.out.println("getWert("+actionCommand+") = "+comboBox.getSelectedItem());
        return comboBox.getSelectedItem();
      }
    }
    if(DEBUG)
      System.out.println("getWert("+actionCommand+") = null");
    return null;
  }

}