package haushalt;
import app.AbstractPrintablePane;
import haushalt.werte.*;

import java.awt.*;
import java.awt.geom.*;

/**
 * @author Lars H. Hahn
 * @version 1.1
 */

public class BalkenPane
  extends AbstractPrintablePane {

  int abstand = 2;
  IntervallSumme[] werte;
  Euro grWert = new Euro();
  Euro schnittEinnahmen;
  Euro schnittAusgaben;

  public BalkenPane(IntervallSumme[] werte) {
    super("Einnahmen/Ausgaben ("+werte[0]+" bis "+werte[werte.length-1]+")");
    this.werte = werte;
    setPreferredSize(new Dimension(500,500));

    Euro summeEinnahmen = new Euro();
    Euro summeAusgaben = new Euro();
    int anzWerte = werte.length;

    for(int i=0; i<anzWerte; i++) {
      if(grWert.compareTo(werte[i].einnahmen) < 0)
        grWert = werte[i].einnahmen;
      if(grWert.compareTo(werte[i].ausgaben) < 0)
        grWert = werte[i].ausgaben;
      summeEinnahmen = summeEinnahmen.add(werte[i].einnahmen);
      summeAusgaben = summeAusgaben.add(werte[i].ausgaben);
    }
    schnittEinnahmen = new Euro(summeEinnahmen.toDouble() / anzWerte);
    schnittAusgaben = new Euro(summeAusgaben.toDouble() / anzWerte);
  }

  public void zeichnen(Graphics g, int fensterX, int fensterY, int fensterBreite, int fensterHoehe) {
    Graphics2D g2 = (Graphics2D) g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    FontMetrics fontMetrics = g2.getFontMetrics();
    int textBreite      = fontMetrics.stringWidth("99.999,99 ");
    int textHoehe       = fontMetrics.getHeight();
    int intervallBreite = (fensterBreite - textBreite) / werte.length;
    int balkenBreite    = intervallBreite * 6 / 15;
    int xEinnahmenText  = intervallBreite * 5 / 15;
    int xEinnahmen      = intervallBreite * 2 / 15;
    int xAusgaben       = intervallBreite * 9 / 15;
    int xAusgabenText   = intervallBreite * 12 / 15;
    int yBalken         = fensterHoehe - textHoehe + fensterY;
    int maxBalkenHoehe  = fensterHoehe - 2 * textHoehe;

    int yLinie;
    for(int y = 0; y <= 8; y++) {
      double faktor = y / 8.0D;
      Euro euro = new Euro(grWert.toDouble()*faktor);
      yLinie = yBalken - (int)(maxBalkenHoehe * faktor);
      g2.setPaint(Color.gray);
      g2.drawLine(textBreite + fensterX, yLinie, fensterBreite + fensterX, yLinie);
      g2.setPaint(Color.black);
      g2.drawString(euro.toString(),
                    fensterX + textBreite - fontMetrics.stringWidth(euro.toString()) - 3,
                    yLinie);
    }
    yLinie = yBalken - (int)(maxBalkenHoehe * schnittEinnahmen.toDouble() / grWert.toDouble());
    g2.setPaint(Color.blue);
    g2.drawRect(textBreite + fensterX, yLinie, fensterBreite-textBreite, 1);
    yLinie = yBalken - (int)(maxBalkenHoehe * schnittAusgaben.toDouble() / grWert.toDouble());
    g2.setPaint(Color.red);
    g2.drawRect(textBreite + fensterX, yLinie, fensterBreite-textBreite, 1);

    int x = textBreite + fensterX;
    for(int i=0; i<werte.length; i++) {
      int hoeheEinnahmen = (int)(maxBalkenHoehe * werte[i].einnahmen.toDouble() / grWert.toDouble());
      g2.setPaint(Color.blue);
      g2.fill3DRect(x+xEinnahmen, yBalken-hoeheEinnahmen, balkenBreite, hoeheEinnahmen, true);

      int hoeheAusgaben = (int)(maxBalkenHoehe * werte[i].ausgaben.toDouble() / grWert.toDouble());
      g2.setPaint(Color.red);
      g2.fill3DRect(x+xAusgaben, yBalken-hoeheAusgaben, balkenBreite, hoeheAusgaben, true);

      g2.setPaint(Color.black);
      g2.drawString(werte[i].toString(), x+xEinnahmen, fensterHoehe + fensterY - 5);

      AffineTransform oldAt = g2.getTransform();
      g2.setPaint(Color.white);
      g2.rotate(Math.toRadians(90),x+xEinnahmenText, yBalken-hoeheEinnahmen+5);
      g2.drawString(werte[i].einnahmen.toString(), x+xEinnahmenText, yBalken-hoeheEinnahmen+5);
      g2.setTransform(oldAt);
      g2.rotate(Math.toRadians(90),x+xAusgabenText, yBalken-hoeheAusgaben+5);
      g2.drawString(werte[i].ausgaben.toString(), x+xAusgabenText, yBalken-hoeheAusgaben+5);
      g2.setTransform(oldAt);

      x += intervallBreite;
    }

  }
}