package haushalt;
import haushalt.werte.*;
import app.Applikation;

import java.io.*;
import java.util.*;

/**
 * @author Lars H. Hahn
 * @version 1.1
 */

abstract public class Buchung
  implements Cloneable, Comparable, Speicherbar {

  private Datum  datum = new Datum();
  private String text  = "";

  abstract public String getBuchungsZeile(Register register);
  abstract public String getBuchungsZeile(Register register, Kategorie kategorie, boolean unterkategorien);

  abstract public Object clone();

  public void setDatum(Datum datum) {
    this.datum = datum;
  }

  public Datum getDatum() {
    return datum;
  }

  public void setText(String text) {
    this.text = text;
  }

  public String getText() {
    return text;
  }

  abstract public void setWert(Euro wert);

  abstract public Euro getWert();

  public boolean sucheText(String text) {
    return (this.text.indexOf(text) > -1);
  }

  public boolean ersetzeText(String alt, String neu) {
    int idx = text.indexOf(alt);
    if(idx != -1) {
      setText(text.substring(0, idx)+neu+text.substring(idx+alt.length()));
      return true;
    }
    return false;
  }

  abstract public void ersetzeKategorie(Kategorie alteKategorie, Kategorie neueKategorie);

  abstract public void summieren(ZeitIntervall intervall);

  static public Buchung importQuicken(InputStream in, boolean dm)
    throws IOException, ImportQuickenException {
    int c;
    c = in.read();
    if(c == -1) // EOF erreicht
      return null;

    // Datum einlesen
    if(c != 'D') {
      Applikation.fehlerAusgabe.println("-E- QuickenImport: Ups! D (Datum) erwartet.");
      return null;
    }
    StringTokenizer st = new StringTokenizer(leseZeile(in), ".");
    String monat = st.nextToken();
    if(monat.length() == 1)
      monat = "0" + monat;
    String tag = st.nextToken();
    if(tag.length() == 1)
      tag = "0" + tag;
    String jahr = st.nextToken();
    Datum datum = new Datum(tag+"."+monat+"."+jahr);

    // Betrag einlesen
    if(in.read() != 'U') {
      Applikation.fehlerAusgabe.println("-E- QuickenImport: Ups! U (Betrag) erwartet.");
      return null;
    }
    Euro einzelWert = new Euro(leseZeile(in).replace('.',',')); // US -> deutsch
    if(in.read() != 'T') {
      Applikation.fehlerAusgabe.println("-E- QuickenImport: Ups! T (Betrag) erwartet.");
      return null;
    }
    if(!einzelWert.equals(new Euro(leseZeile(in).replace('.',',')))) {
      Applikation.fehlerAusgabe.println("-E- QuickenImport: Ups! U und T Betrag ungleich.");
      return null;
    }
    if(dm)
      einzelWert.umrechnenVonDM();

    // Abgeglichene Buchung "X" einlesen
    c = in.read();
    if(c == 'C') {
      leseZeile(in);
      c = in.read();
    }

    // Buchungstext einlesen
    String text = "*";
    if(c == 'P') {
      text = leseZeile(in);
      c = in.read();
    }

    // Kategorie einlesen
    String einzelKategorie;
    if(c == 'L') {
      einzelKategorie = leseZeile(in);
      c = in.read();
    }
    else {
      einzelKategorie = Kategorie.SONSTIGES.toString();
      Applikation.fehlerAusgabe.println("-I- Buchung ohne Kategorie -> 'Sonstiges'");
    }

    // Split-Buchungen einlesen
    ImportQuickenException e = new ImportQuickenException();
    boolean umbuchungInSplitBuchung = false;
    SplitBetrag splitBetrag = null;
    while(c == 'S') {
      String kategorie = leseZeile(in);
      if(in.read() != '$') {
        Applikation.fehlerAusgabe.println("-E- QuickenImport: Ups! $ (Betrag-Splitbuchung) erwartet.");
        return null;
      }
      Euro wert = new Euro(leseZeile(in).replace('.',',')); // US -> deutsch
      if(dm)
        wert.umrechnenVonDM();
      if((kategorie.length() > 2) &&
          kategorie.startsWith("[") &&
          kategorie.endsWith("]")) {
        umbuchungInSplitBuchung = true;
        // Problem: Das Ziel-Register der Umbuchung gibt es vielleicht noch nicht:
        if(Register.findeRegister(kategorie) == null)  // also erzeugen!
          Register.erzeugeRegister(kategorie);
        e.addBuchung(new Umbuchung(datum, text, kategorie, wert));
      }
      else {
        if(splitBetrag == null)
          splitBetrag = new SplitBetrag(kategorie, wert);
        else
          splitBetrag.add(kategorie, wert);
      }
      c = in.read();
    }

    if(c != '^') {
      Applikation.fehlerAusgabe.println("-E- QuickenImport: Ups! ^ (Datensatzende) erwartet.");
      return null;
    }
    leseZeile(in); // CR+LF von '^' lesen
    if(umbuchungInSplitBuchung) {
      if(splitBetrag != null) // falls gemischte Splitbuchung
        e.addBuchung(new StandardBuchung(datum, text, splitBetrag));
      throw e;
    }

    if(splitBetrag == null) {
      if((einzelKategorie.length() > 2) &&    // Pruefen ob Umbuchung
          einzelKategorie.startsWith("[") &&
          einzelKategorie.endsWith("]")) {
        // Problem: Das Ziel-Register der Umbuchung gibt es vielleicht noch nicht:
        if(Register.findeRegister(einzelKategorie) == null)  // also erzeugen!
          Register.erzeugeRegister(einzelKategorie);
        return new Umbuchung(datum, text, einzelKategorie, einzelWert);
      }
      splitBetrag = new SplitBetrag(einzelKategorie, einzelWert);
    }
    return new StandardBuchung(datum, text, splitBetrag);
  }

  public static class ImportQuickenException
    extends Exception {

    public ArrayList buchungList = new ArrayList();

    public void addBuchung(Buchung buchung) {
      buchungList.add(buchung);
    }

    public Buchung getBuchung() {
      if(buchungList.isEmpty())
        return null;
      else
        return (Buchung)buchungList.remove(0);
    }
  }

  public static String leseZeile(InputStream in)
    throws IOException {
    String zeile = "";
    int c;
    while((c = in.read()) != 13) { // Bis zum Zeilenende (CR)
      if(c == -1) {
        Applikation.fehlerAusgabe.println("-E- QuickenImport: Ups! Unerwartetes EOF.");
        return "";
      }
      if(c != ',') // Kommata werden gefiltert; Problem beim Float-Parsen
        zeile = zeile + (char)c;
    }
    if((c = in.read()) != 10) {
      Applikation.fehlerAusgabe.println("-E- QuickenImport: Ups! Nach CR kam kein LF. "+(char)c);
    }
    return zeile;
  }

  /*
   * Methoden des Interface 'Comparable'
   */

  public int compareTo(Object obj) {
    if(obj.getClass() == Datum.class)
      return this.getDatum().compareTo(obj);
    if((obj.getClass() == StandardBuchung.class) || (obj.getClass() == Umbuchung.class))
      return this.getDatum().compareTo(((Buchung)obj).datum);
   Applikation.fehlerAusgabe.println("-E- Buchung: Fataler Fehler beim Vergleich: "+obj.getClass());
   return -1;
  }

  /*
   * Methoden des Interface 'Speicherbar'
   */

  public void laden(DataInputStream in)
    throws IOException {
    datum.laden(in);
    text = in.readUTF();
  }

  public void speichern(DataOutputStream out)
    throws IOException {
    datum.speichern(out);
    out.writeUTF(text);
  }

}