package haushalt;
import haushalt.werte.*;
import app.Applikation;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * @author Lars H. Hahn
 * @version 1.1
 */


/* Problem/Loesung: Die uebergeordnete Tabelle greift mittelbar den Inhalt aus
 * dem 'Document' des TextField ab. Wenn das TextFeld nicht den Focus erhaelt,
 * kann es auch kein 'ActionEvent' beim Verlieren des Focus liefern!
 * Die Daten kommen mittelbar ber die 'insertString'- und 'getText'-Funktion
 * des 'Document' von/in die uebergeordnete Tabelle.
 * ==> KEINEN ActionListener oder FocusListener beim TextField zum Abgreifen
 *     seines Wertes verwenden!
 */

public class BuchungsEditor
  extends DefaultCellEditor
  implements KeyListener {

  private final static boolean DEBUG = false;

  private static int deltaste = 0;
  private Buchung buchung;
  private JFrame masterFrame;

  public BuchungsEditor(JFrame masterFrame) {
    super(new JCheckBox("Dummy"));
    this.masterFrame = masterFrame;
    setClickCountToStart(1);
  }

  public static void setDeltaste(String text) {
    if(text.equals("Shift-Entf"))
      deltaste = InputEvent.SHIFT_MASK;
    else if(text.equals("Strg-Entf"))
      deltaste = InputEvent.CTRL_MASK;
    else if(text.equals("Alt-Entf"))
      deltaste = InputEvent.ALT_MASK;
    else
      deltaste = 0;
  }

  public Object getCellEditorValue() {
    return ((BuchungsField)editorComponent).aktualisiereBuchung();
  }

  public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int col) {
    buchung = (Buchung)value;
    if(value.getClass() == StandardBuchung.class) {
      StandardBuchung standardBuchung = (StandardBuchung)value;
      if(standardBuchung.getSplitBetrag().isSplit() && ((col == 2) || (col == 3))) {
        splitBetragBearbeiten(standardBuchung.getSplitBetrag());
        return null; // wenn das TextField zurueckgegeben wird, wird es auch verwendet!
      }
      switch(col) {
        case 0: editorComponent = new BuchungsDatumField(standardBuchung); break;
        case 1: editorComponent = new BuchungstextField(standardBuchung, masterFrame); break;
        case 2: editorComponent = new BuchungsComboBox(standardBuchung); break;
        case 3: editorComponent = new BuchungsEuroField(standardBuchung); break;
      }
    }

    if(value.getClass() == Umbuchung.class) {
      Umbuchung umbuchung = (Umbuchung)value;
      Register register = (Register)table.getModel();
      switch(col) {
        case 0: editorComponent = new BuchungsDatumField(umbuchung); break;
//        case 1: return null; // Umbuchungen nicht editieren!
        case 1: editorComponent = new BuchungstextField(umbuchung, masterFrame); break;
        case 2: editorComponent = new BuchungsComboBox(umbuchung, register); break;
        case 3: editorComponent = new BuchungsEuroField(umbuchung); break;
      }
    }
    editorComponent.addKeyListener(this);
    return editorComponent;
  }

  private void splitBetragBearbeiten(SplitBetrag splitBetrag) {
    // Diaglog fuer den SplitEditor vorbereiten:
    final JDialog dlg = new JDialog(masterFrame, "Split-Betrag Editor", true);
    final JPanel pane = (JPanel)dlg.getContentPane();
    final JButton buttonFertig = new JButton("Fertig");
    buttonFertig.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dlg.setVisible(false);
      }
    });
    // wesentlicher Inhalt ist eine Tabelle aus dem SplitBetrag generiert
    JTable subTable = new JTable(splitBetrag);
    subTable.addKeyListener(splitBetrag);
    // Editor fuer die Kategorien
    subTable.setDefaultEditor(Kategorie.class,
      new DefaultCellEditor(new JComboBox(Kategorie.getKategorien(true))));
    // Editor fuer den Betrag (Euro)
    DefaultCellEditor euroEditor = new DefaultCellEditor(new EuroField(Euro.NULL_EURO)) {
      public Object getCellEditorValue() {
        return ((EuroField)editorComponent).getValue();
      }
      public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int col) {
        if(value.getClass() == String.class)
          editorComponent = new EuroField(new Euro((String)value));
        else
          editorComponent = new EuroField((Euro)value);
        return editorComponent;
      }
    };
    subTable.setDefaultEditor(Euro.class, euroEditor);
    subTable.setPreferredScrollableViewportSize(new Dimension(500, 140));

    pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
    pane.add(new JScrollPane(subTable), BorderLayout.CENTER);
    pane.add(buttonFertig, BorderLayout.SOUTH);
    dlg.pack();
    dlg.setVisible(true);
    // Tricky-Workaround:
    // Wenn eine Nicht-Splitbuchung gesplittet werden soll, ist die editorComponent
    // schon gesetzt. Wenn die editorComponent waehrend des folgenden stopEditing mit
    // editorComponent.aktuallisiereBuchung aufgerufen wird, kommt es beim EuroField und
    // bei der ComboBox zum Fehler. stopEditing ist in jedem Fall noetig, sonst bleibt
    // die (falsche) editorComponent in der Table stehen. Deshalb wird die editorComponent
    // auf das (unkritische) DatumField gesetzt.
    editorComponent = new BuchungsDatumField((StandardBuchung)buchung);
    this.stopCellEditing();
  }

  /*
   * Methoden des Interface 'KeyListener'
   */

  public void keyReleased(KeyEvent e) {
    // weitere Tasten-Auswertungen in Register
    if(DEBUG)
      System.out.println("BuchungsEditor: KeyCode="+e.getKeyCode());

    if((e.getModifiers() == 8) && (e.getKeyCode() == 'S')) { // Alt-S
      if(buchung.getClass() == StandardBuchung.class) {
        StandardBuchung standardBuchung = (StandardBuchung)buchung;
        splitBetragBearbeiten(standardBuchung.getSplitBetrag());
      }
      else
        JOptionPane.showMessageDialog(masterFrame,
          "Umbuchung knnen nicht gesplittet werden!",
          "Alt-S: Split-Buchung",
          JOptionPane.WARNING_MESSAGE);
    }

    if((e.getModifiers() == deltaste) && (e.getKeyCode() == 127)) {
      if((e.getSource().getClass() == BuchungsDatumField.class) ||
         (e.getSource().getClass() == BuchungstextField.class) ||
         (e.getSource().getClass() == BuchungsEuroField.class))
        ((JTextField)e.getSource()).setText("");
      else if(e.getSource().getClass() == JTable.class) {
        JTable table = (JTable)e.getSource();
        if (table.getEditingColumn() != 2) {
          JTextField textField =(JTextField)table.getEditorComponent();
          if(textField == null)
            JOptionPane.showMessageDialog(masterFrame,
              "Buchungstext einer Umbuchung und der Saldo knnen nicht gelscht werden!",
              "DEL: Split-Buchung",
              JOptionPane.WARNING_MESSAGE);
          else
            textField.setText("");
        }
      }
      else
        Applikation.fehlerAusgabe.println("-E- Ups! Unerwartete KeyEvent-Quelle (DEL):"+e.getSource());
    }

  }

  public void keyPressed(KeyEvent e){};

  public void keyTyped(KeyEvent e){};
}