package haushalt;
import haushalt.werte.*;
import app.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import java.io.*;
import java.util.*;
import java.lang.Double;
import javax.swing.*;
import javax.swing.filechooser.FileSystemView;
import java.net.URL;

/**
 * @author Lars H. Hahn
 * @version 1.1
 */

public class Haushalt
 extends Applikation {

  private final static boolean DEBUG = false;

  final private String[][] menuDatei = {
    {"DATEI", "Datei"},
    {"neu", "Neu"},
    {"laden", "Laden ..."},
    {"#_speichern", "Speichern"},
    {"#_speichernUnter", "Speichern unter ..."},
    {"-", "-"},
    {"#_importQuicken", "Import QIF ..."},
    {"X_euroImport", "Importwhrung Euro"},
    {"seiteEinrichten", "Seite einrichten..."},
    {"beenden", "Beenden"}
  };
  final private String[][] menuAuswertung = {
    {"AUSWERTUNG", "Auswertung"},
    {"#_balkenDiagramm", "Balkendiagramm"},
    {"#_tortenDiagramm", "Tortendiagramm"},
    {"#_vermoegensentwicklung", "Vermgensentwicklung"},
    {"-", "-"},
    {"#_einnahmenAusgaben", "Einnahmen & Ausgaben"},
    {"#_kategorieSummen", "Kategorie-Summen"},
    {"#_absolutVergleich", "Vergleich Zeitrume (Absolut)"},
    {"#_schnittVergleich", "Vergleich Zeitrume (Durchschnitt)"},
    {"#_vermoegensuebersicht", "Vermgensbersicht"},
    {"-", "-"},
    {"X_DRUCKER", "Ausgabe auf den Drucker"}
  };
  final private String[][] menuBearbeiten = {
    {"BEARBEITEN", "Bearbeiten"},
    {"#_suchen", "Suchen und Ersetzen ..."},
    {"#_umbuchen", "Umbuchung eingeben ..."},
    {"neuesRegister", "Neues Register ..."},
    {"neueKategorie", "Neue Kategorie ..."},
    {"-", "-"},
    {"#_alteBuchungenLoeschen", "alte Buchungen entfernen ..."},
    {"#_kategorieErsetzen", "Kategorie ersetzen ..."},
    {"#_registerVereinigen", "Register vereinigen ..."}
  };
  final private String[][] menuExtras = {
    {"EXTRAS", "Extras"},
    {"optionen", "Optionen ..."},
    {"#_autoBuchung", "Automatische Buchungen bearbeiten ..."},
    {"#_registerAnzeigen", "Ausgewhlte Buchungen (Register) ..."},
    {"#_kategorieAnzeigen", "Ausgewhlte Buchungen (Kategorie) ..."},
    {"-", "-"},
    {"performanzTest", "Performanz-Test ..."}
  };
  final private String[][] menuHilfe = {
    {"HILFE", "Hilfe"},
    {"hilfeInhalt", "Inhalt ..."},
    {"-", "-"},
    {"programmInfo", "Programm Informationen ..."},
  };

  private boolean unterkat;
  final private DlgSuchenErsetzen dlgSuchenErsetzen;
  final private AutoBuchung autoBuchung = new AutoBuchung();
  final private ProgressMonitor monitor = new ProgressMonitor(
    getOberflaecheHaushalt(),
    "Bitte warten ...",
    "Initialisierung",
    0, 100);

  public Haushalt(OberflaecheHaushalt oberflaeche) {
    super(oberflaeche);
    monitor.setMillisToDecideToPopup(0);
    monitor.setMillisToPopup(250);
    name = "jHaushalt";
    kuerzel = "jhh";
    version = "1.1";
    copyright = "jHaushalt v1.1 * (C)opyright 2002 Lars H. Hahn";
    jarDateiname = "jHaushalt.jar";
    oberflaeche.addHintergrund(new JLabel(new ImageIcon(bildLaden("jhh.gif"))));
    if(DEBUG)
      System.out.println("Hintergrund geladen.");
    getMenuHandler().addMenu(menuDatei);
    getMenuHandler().addMenu(menuAuswertung);
    getMenuHandler().addMenu(menuBearbeiten);
    getMenuHandler().addMenu(menuExtras);
    getMenuHandler().addMenu(menuHilfe);
    getMenuHandler().setzeTaste("neu", KeyEvent.VK_N);
    getMenuHandler().setzeTaste("laden", KeyEvent.VK_L);
    getMenuHandler().setzeTaste("#_speichern", KeyEvent.VK_S);
    getMenuHandler().setzeTaste("beenden", KeyEvent.VK_X);
    getMenuHandler().setzeTaste("optionen", KeyEvent.VK_O);
    getMenuHandler().setzeTaste("hilfeInhalt", KeyEvent.VK_F1);
    getMenuHandler().setzeTaste("#_umbuchen", KeyEvent.VK_U);
    if(DEBUG)
      System.out.println("Men initialisiert.");
    super.init();
    unterkat = Boolean.valueOf(properties.getProperty(kuerzel+".opt.unterkat", "false")).booleanValue();
    boolean gemerkte = Boolean.valueOf(properties.getProperty(kuerzel+".opt.gemerkte", "true")).booleanValue();
    Register.setGemerkteBuchungenVerwenden(gemerkte);
    String reiter = properties.getProperty(kuerzel+".opt.reiter", "BOTTOM");
    oberflaeche.setTabPlacement(reiter);
    dlgSuchenErsetzen = new DlgSuchenErsetzen(oberflaeche);
    if(DEBUG)
      System.out.println("Applikation initialisiert.");
  }

  public OberflaecheHaushalt getOberflaecheHaushalt() {
    return (OberflaecheHaushalt)super.getOberflaeche();
  }

// === menuDatei ===============================================================

// --- neu ---------------------------------------------------------------------
  public boolean neu() {
    if(super.neu()) {
      autoBuchung.allesLoeschen();
      Register.allesLoeschen();
      Kategorie.allesLoeschen();
      getMenuHandler().setSelectedEnabled(false);
      getOberflaecheHaushalt().zeigeHintergrund();
      return true;
    }
    return false;
  }

// --- laden -------------------------------------------------------------------
  protected void laden(DataInputStream in)
    throws IOException {
    monitor.setNote("0 Register geladen.");
    monitor.setProgress(10);
    String versionInfo = in.readUTF();
    if(!versionInfo.equals(name+version)) {
      fehlerAusgabe.print("-W- Falsche Version beim Laden! ");
      fehlerAusgabe.println("Gefunden: "+versionInfo+", Erwartet: "+name+version);
    }
    int size = in.readInt();
    Register[] registers = new Register[size];
    for(int i=0; i<size; i++) {
      String name = in.readUTF();              // Dirty: nicht sysmetrisch!
      registers[i] = Register.findeRegister(name); // Register kann schon durch eine
      if(registers[i] == null)                     // Umbuchung erzeugt sein!
        registers[i] = Register.erzeugeRegister(name);
      registers[i].laden(in);
      monitor.setProgress(10 + 80*(i+1)/size);
      monitor.setNote(""+(i+1)+" Register geladen.");
    }
    // automatische Buchungen laden und ausfhen
    if(!versionInfo.equals("jHaushalt1.0")) {
      autoBuchung.laden(in);
      int anzahl = autoBuchung.ausfuehren();
      if(anzahl > 0)
        fehlerAusgabe.println("-I- Es wurden "+anzahl+" automatische Buchung ausgefhrt.");
    }
    else
      fehlerAusgabe.println("-I- Daten von Version 1.0 erfolgreich konvertiert.");
    getMenuHandler().setSelectedEnabled(true);
    getOberflaecheHaushalt().zeigeAlleRegister();
    monitor.setProgress(100);
  }

// --- speichern ---------------------------------------------------------------
  protected void speichern(DataOutputStream out)
    throws IOException {
    Register[] register = Register.getRegister();
    out.writeUTF(name+version);
    out.writeInt(register.length);
    for(int i=0; i<register.length; i++)
      register[i].speichern(out);
    autoBuchung.speichern(out);
    out.flush();
  }

// --- importQuicken -----------------------------------------------------------
  public void importQuicken() {
    DlgMaster dlg = new DlgMaster("QIF Import", getOberflaecheHaushalt());
    PaneKategorie pane = new PaneKategorie("Bitte whlen Sie ein Register aus:",
                                            Register.getRegisterNamen());
    dlg.addPane(pane);

    if(Register.getRegister().length == 0)
      JOptionPane.showMessageDialog(getOberflaecheHaushalt(), "Ups! Noch kein Register vorhaden!");
    else if(dlg.showDialog()) {
      Register register = Register.findeRegister((String)pane.getWert());
      JFileChooser dateidialog = new JFileChooser();
      dateidialog.setCurrentDirectory(new File(properties.getProperty(kuerzel+".ordner")));
      if(dateidialog.showOpenDialog(getOberflaecheHaushalt()) == JFileChooser.APPROVE_OPTION) {
        File datei = dateidialog.getSelectedFile();
        try {
          FileInputStream in = new FileInputStream(datei);
          register.importQuicken(in, getMenuHandler().getState("X_euroImport"));
          in.close();
          getOberflaecheHaushalt().zeigeAlleRegister();
        }
        catch(FileNotFoundException e1) {
          fehlerAusgabe.println("-E- "+datei.getPath()+" nicht gefunden!");
        }
        catch(IOException e2) {
          fehlerAusgabe.println("-E- Fehler beim Importieren: "+datei.getPath());
        }
      }
    }
  }

// === menuAuswertung ==========================================================

// --- balkenDiagramm ----------------------------------------------------------
  public void balkenDiagramm() {
    final String ueberschrift = "Balkendiagramm";
    DlgMaster dlg = new DlgMaster(ueberschrift, getOberflaecheHaushalt());
    PaneIntervall pane1 = new PaneIntervall("Erster Zeitraum:");
    dlg.addPane(pane1);
    PaneZahl pane2 = new PaneZahl("Anzahl Zeitrume:", new Integer(4));
    dlg.addPane(pane2);

    if(dlg.showDialog()) {
      ZeitIntervall intervall = (ZeitIntervall)pane1.getWert();
      int anzIntervalle = ((Integer)pane2.getWert()).intValue();
      IntervallSumme[] intervallSummen = new IntervallSumme[anzIntervalle];
      for(int i=0; i<anzIntervalle; i++) {
        intervallSummen[i] = new IntervallSumme(intervall.toString());
        intervallSummen[i].sum(Register.summieren(intervall, unterkat));
        intervall = intervall.folgeIntervall();
      }
      BalkenPane balkenPane = new BalkenPane(intervallSummen);
      anzeigenOderDrucken(ueberschrift, balkenPane);
    }
  }

// --- tortenDiagramm ----------------------------------------------------------
  public void tortenDiagramm() {
    int maxWerteTortenDiagramm = Integer.parseInt(properties.getProperty(kuerzel+".opt.maxtorte", "10"));
    final String ueberschrift = "Tortendiagramm";
    DlgMaster dlg = new DlgMaster(ueberschrift, getOberflaecheHaushalt());
    PaneIntervall pane = new PaneIntervall("Zeitraum:");
    dlg.addPane(pane);

    if(dlg.showDialog()) {
      final ZeitIntervall intervall = (ZeitIntervall)pane.getWert();
      String[] kategorie = Kategorie.getKategorien(unterkat);
      int anzKat = kategorie.length;
      TortenWert[] einnahmen = new TortenWert[maxWerteTortenDiagramm];
      TortenWert[] ausgaben = new TortenWert[maxWerteTortenDiagramm];
      Euro gesamtEinnahmen = Euro.NULL_EURO;
      Euro gesamtAusgaben = Euro.NULL_EURO;
      Euro[] summe = Register.summieren(intervall, unterkat);

      for(int i=0; i<anzKat; i++) {
        if(summe[i].compareTo(Euro.NULL_EURO)>0) { // Einnahmen:
          gesamtEinnahmen = gesamtEinnahmen.add(summe[i]);
          TortenWert neuerWert = new TortenWert(summe[i], kategorie[i]);
          for(int j=0; j<maxWerteTortenDiagramm; j++) {
            if(einnahmen[j] == null) {
              einnahmen[j] = neuerWert; // Prima! Noch ein Platz frei!
              j = maxWerteTortenDiagramm;
            }
            else if(neuerWert.groesser(einnahmen[j])) {
              for(int k=maxWerteTortenDiagramm-1; k>j; k--)
                einnahmen[k] = einnahmen[k-1];
              einnahmen[j] = neuerWert;
              j = maxWerteTortenDiagramm;
            }
          }
        }
        else { // Ausgaben:
          gesamtAusgaben = gesamtAusgaben.sub(summe[i]);
          TortenWert neuerWert = new TortenWert(Euro.NULL_EURO.sub(summe[i]), kategorie[i]);
          for(int j=0; j<maxWerteTortenDiagramm; j++) {
            if(ausgaben[j] == null) {
              ausgaben[j] = neuerWert; // Prima! Noch ein Platz frei!
              j = maxWerteTortenDiagramm;
            }
            else if(neuerWert.groesser(ausgaben[j])) {
              for(int k=maxWerteTortenDiagramm-1; k>j; k--)
                ausgaben[k] = ausgaben[k-1];
              ausgaben[j] = neuerWert;
              j = maxWerteTortenDiagramm;
            }
          }
        }
      }
      TortenPane tortenPane = new TortenPane(
          "Verteilung Einnahmen/Ausgaben ("+intervall+")",
          properties.getProperty(kuerzel+".opt.farbschema", TortenPane.schemaNamen[0]),
          einnahmen, gesamtEinnahmen, ausgaben, gesamtAusgaben);
      anzeigenOderDrucken(ueberschrift, tortenPane);
   }
  }

// --- vermoegensentwicklung ---------------------------------------------------
  public void vermoegensentwicklung() {
    final String ueberschrift = "Vermgensentwicklung";
    DlgMaster dlg = new DlgMaster(ueberschrift, getOberflaecheHaushalt());
    PaneIntervall pane1 = new PaneIntervall("Erster Zeitraum:");
    dlg.addPane(pane1);
    PaneZahl pane2 = new PaneZahl("Anzahl Zeitrume:", new Integer(4));
    dlg.addPane(pane2);

    if(dlg.showDialog()) {
      ZeitIntervall intervall = (ZeitIntervall)pane1.getWert();
      int anzIntervalle = ((Integer)pane2.getWert()).intValue() + 1;
      Datum[] zeitpunkte = new Datum[anzIntervalle];
      Euro[] salden = new Euro[anzIntervalle];
      for(int i=0; i<anzIntervalle; i++) {
        Datum datum = intervall.getStartDatum();
        zeitpunkte[i] = datum;
        salden[i] = Register.getGesamtSaldo(datum);
        intervall = intervall.folgeIntervall();
      }
      VermoegenPane vermoegenPane = new VermoegenPane(zeitpunkte, salden);
      anzeigenOderDrucken(ueberschrift, vermoegenPane);
    }
  }

// --- einnahmenAusgaben -------------------------------------------------------
  public void einnahmenAusgaben() {
    final String ueberschrift = "Einnahmen/Ausgaben";
    DlgMaster dlg = new DlgMaster(ueberschrift, getOberflaecheHaushalt());
    PaneIntervall pane1 = new PaneIntervall("Erster Zeitraum:");
    dlg.addPane(pane1);
    PaneZahl pane2 = new PaneZahl("Anzahl Zeitrume:", new Integer(4));
    dlg.addPane(pane2);

    if(dlg.showDialog()) {
      ZeitIntervall intervall = (ZeitIntervall)pane1.getWert();
      int anzIntervalle = ((Integer)pane2.getWert()).intValue();
      IntervallSumme[] intervallSummen = new IntervallSumme[anzIntervalle];
      for(int i=0; i<anzIntervalle; i++) {
        intervallSummen[i] = new IntervallSumme(intervall.toString());
        intervallSummen[i].sum(Register.summieren(intervall, unterkat));
        intervall = intervall.folgeIntervall();
      }

      Object[][] data = new Object[anzIntervalle+2][4];
      String kopf[] = {"Zeitraum", "Einnahmen","Ausgaben","berschu"};
      data[0] = kopf;
      Euro summeEinnahmen = new Euro();
      Euro summeAusgaben = new Euro();
      for(int i=0; i<anzIntervalle; i++) {
        data[i+1][0] = intervallSummen[i].toString();
        data[i+1][1] = intervallSummen[i].einnahmen;
        data[i+1][2] = intervallSummen[i].ausgaben;
        data[i+1][3] = intervallSummen[i].gesamt();
        summeEinnahmen = summeEinnahmen.add(intervallSummen[i].einnahmen);
        summeAusgaben = summeAusgaben.add(intervallSummen[i].ausgaben);
      }
      data[anzIntervalle+1][0] = "Durchschnitt";
      data[anzIntervalle+1][1] = summeEinnahmen.durch(anzIntervalle);
      data[anzIntervalle+1][2] = summeAusgaben.durch(anzIntervalle);
      data[anzIntervalle+1][3] = summeEinnahmen.add(summeAusgaben).durch(anzIntervalle);
      TabellenPane tabellenPane =
          new TabellenPane(ueberschrift+" ("+intervallSummen[0]+" bis "+intervallSummen[anzIntervalle-1]+")", data);
      anzeigenOderDrucken(ueberschrift, tabellenPane);
    }
  }

// --- kategorieSummen ---------------------------------------------------------
  public void kategorieSummen() {
    final String ueberschrift = "Kategorie Summen";
    DlgMaster dlg = new DlgMaster(ueberschrift, getOberflaecheHaushalt());
    PaneIntervall pane1 = new PaneIntervall("Erster Zeitraum:");
    dlg.addPane(pane1);
    PaneZahl pane2 = new PaneZahl("Anzahl Zeitrume:", new Integer(4));
    dlg.addPane(pane2);

    if(dlg.showDialog()) {
      ZeitIntervall intervall  = (ZeitIntervall)pane1.getWert();
      int anzIntervalle        = ((Integer)pane2.getWert()).intValue();
      String[] kategorien      = Kategorie.getKategorien(unterkat);
      int anzKategorien        = kategorien.length;
      Euro[][] summen          = new Euro[anzIntervalle][anzKategorien];
      Object[][] data          = new Object[anzKategorien+2][anzIntervalle+1];
      data[0][0]               = "Kategorie";
      data[anzKategorien+1][0] = "S U M M E";

      for(int i=0; i<anzIntervalle; i++) {
        data[0][i+1] = intervall.toString();
        summen[i] = Register.summieren(intervall, unterkat);
        intervall = intervall.folgeIntervall();
      }

      for(int j=0; j<anzIntervalle; j++)
        data[anzKategorien+1][j+1] = new Euro();
      for(int i=0;i<anzKategorien;i++) {
        data[i+1][0] = kategorien[i];
        for(int j=0; j<anzIntervalle; j++) {
          data[i+1][j+1] = summen[j][i];
          data[anzKategorien+1][j+1] = ((Euro)(data[anzKategorien+1][j+1])).add(summen[j][i]);
        }
      }
    TabellenPane tabellenPane =
        new TabellenPane(ueberschrift+" ("+data[0][1]+" bis "+data[0][anzIntervalle]+")", data);
    anzeigenOderDrucken(ueberschrift, tabellenPane);
    }
  }

// --- absolutVergleich --------------------------------------------------------
  public void absolutVergleich() {
    final String ueberschrift = "Vergleich Zeitrume (Absolut)";
    DlgMaster dlg = new DlgMaster(ueberschrift, getOberflaecheHaushalt());
    PaneIntervall pane = new PaneIntervall("Erster Zeitraum:");
    dlg.addPane(pane);

    if(dlg.showDialog()) {
      ZeitIntervall intervall1 = (ZeitIntervall)pane.getWert();
      ZeitIntervall intervall2 = intervall1.folgeIntervall();
      String[] kategorien = Kategorie.getKategorien(unterkat);
      int anzKat = kategorien.length;
      Euro[] summen1 = Register.summieren(intervall1, unterkat);
      Euro[] summen2 = Register.summieren(intervall2, unterkat);
      Object[][] data          = new Object[anzKat+2][4];
      data[0][0]               = "Kategorie";
      data[0][1]               = intervall1.toString();
      data[0][2]               = intervall2.toString();
      data[0][3]               = "Differenz";
      data[anzKat+1][0]        = "S U M M E";

      for(int i=1; i<4; i++)
        data[anzKat+1][i] = new Euro();
      for(int i=0;i<anzKat;i++) {
        data[i+1][0] = kategorien[i];
        data[i+1][1] = summen1[i];
        data[i+1][2] = summen2[i];
        data[i+1][3] = summen1[i].sub(summen2[i]);
        data[anzKat+1][1] = ((Euro)(data[anzKat+1][1])).add(summen1[i]);
        data[anzKat+1][2] = ((Euro)(data[anzKat+1][2])).add(summen2[i]);
      }
      data[anzKat+1][3] = ((Euro)(data[anzKat+1][1])).sub((Euro)(data[anzKat+1][2]));
      TabellenPane tabellenPane =
          new TabellenPane(ueberschrift, data);
      anzeigenOderDrucken(ueberschrift, tabellenPane);
    }
  }

// --- schnittVergleich --------------------------------------------------------
  public void schnittVergleich() {
    final String ueberschrift = "Vergleich Zeitrume (Durchschnitt)";
    DlgMaster dlg = new DlgMaster(ueberschrift, getOberflaecheHaushalt());
    PaneIntervall pane1 = new PaneIntervall("Erster Zeitraum:");
    dlg.addPane(pane1);
    PaneIntervall pane2 = new PaneIntervall("Zweiter Zeitraum:");
    dlg.addPane(pane2);

    if(dlg.showDialog()) {
      ZeitIntervall intervall1 = (ZeitIntervall)pane1.getWert();
      ZeitIntervall intervall2 = (ZeitIntervall)pane2.getWert();
      double faktor1           = 30.0D / intervall1.getAnzahlTage();
      double faktor2           = 30.0D / intervall2.getAnzahlTage();
      String[] kategorien      = Kategorie.getKategorien(unterkat);
      int anzKat               = kategorien.length;
      Euro[] summen1           = Register.summieren(intervall1, unterkat);
      Euro[] summen2           = Register.summieren(intervall2, unterkat);
      Object[][] data          = new Object[anzKat+2][4];
      data[0][0]               = "Kategorie";
      data[0][1]               = intervall1.toString();
      data[0][2]               = intervall2.toString();
      data[0][3]               = "Differenz";
      data[anzKat+1][0]        = "S U M M E";

      for(int i=1; i<4; i++)
        data[anzKat+1][i] = new Euro();
      for(int i=0;i<anzKat;i++) {
        data[i+1][0] = kategorien[i];
        data[i+1][1] = new Euro(faktor1 * summen1[i].toDouble());
        data[i+1][2] = new Euro(faktor2 * summen2[i].toDouble());;
        data[i+1][3] = ((Euro)data[i+1][1]).sub((Euro)data[i+1][2]);
        data[anzKat+1][1] = ((Euro)data[anzKat+1][1]).add((Euro)data[i+1][1]);
        data[anzKat+1][2] = ((Euro)data[anzKat+1][2]).add((Euro)data[i+1][2]);
      }
      data[anzKat+1][3] = ((Euro)(data[anzKat+1][1])).sub((Euro)(data[anzKat+1][2]));
      TabellenPane tabellenPane =
          new TabellenPane(ueberschrift, data);
      anzeigenOderDrucken(ueberschrift, tabellenPane);
    }
  }

// --- vermoegensuebersicht ----------------------------------------------------
  public void vermoegensuebersicht() {
    final String ueberschrift = "Vermgensbersicht";
    DlgMaster dlg = new DlgMaster(ueberschrift, getOberflaecheHaushalt());
    PaneDatum pane = new PaneDatum("Bitte geben Sie ein Datum ein:", new Datum());
    dlg.addPane(pane);
    if(dlg.showDialog()) {
      Datum datum = (Datum)pane.getWert();
      Register[] register = Register.getRegister();
      String[][] data = new String[register.length+1][2];
      Euro summe = new Euro();
      for(int i=0; i<register.length; i++) {
        Euro saldo = register[i].getSaldo(datum);
        data[i][0] = register[i].toString();
        data[i][1] = saldo.toString();
        summe = summe.add(saldo);
      }
      data[register.length][0] = "SUMME";
      data[register.length][1] = summe.toString();
      TabellenPane tabellenPane =
          new TabellenPane(ueberschrift+" am "+datum, data);
      anzeigenOderDrucken(ueberschrift, tabellenPane);
    }
  }

// === menuBearbeiten ==========================================================

// --- suchen ------------------------------------------------------------------
  public void suchen() {
    dlgSuchenErsetzen.showDialog();
  }

// --- umbuchen ----------------------------------------------------------------
  public void umbuchen() {
    DlgMaster dlg = new DlgMaster("Umbuchung eingeben", getOberflaecheHaushalt());
    PaneDatum pane1 = new PaneDatum("Bitte geben Sie ein Datum ein:", new Datum());
    dlg.addPane(pane1);
    PaneKategorie pane2 = new PaneKategorie("Bitte whlen Sie ein Quell-Register aus:", Register.getRegisterNamen());
    dlg.addPane(pane2);
    PaneKategorie pane3 = new PaneKategorie("Bitte whlen Sie ein Ziel-Register aus:", Register.getRegisterNamen());
    dlg.addPane(pane3);
    PaneEuro pane4 = new PaneEuro("Bitte geben Sie ein Betrag ein:", new Euro());
    dlg.addPane(pane4);
    if(dlg.showDialog()) {
      Datum datum = (Datum)pane1.getWert();
      String quelle = pane2.getWert().toString();
      Euro euro = (Euro)pane4.getWert();
      Umbuchung umbuchung = new Umbuchung(datum, "<Umbuchung>", quelle, euro);
      Register zielRegister = Register.findeRegister(pane3.getWert().toString());
      zielRegister.einfuegenBuchung(umbuchung);
    }
  }

// --- neuesRegister -----------------------------------------------------------
  public void neuesRegister() {
    DlgMaster dlg = new DlgMaster("Neues Register", getOberflaecheHaushalt());
    PaneText pane = new PaneText("Name des neuen Registers:", "NEU");
    dlg.addPane(pane);

    if(dlg.showDialog()) {
      getOberflaecheHaushalt().zeigeRegister(Register.erzeugeRegister("["+pane.getWert()+"]"));
      getMenuHandler().setSelectedEnabled(true); // knnte das erste Register sein -> enable
    }
  }

// --- neueKategorie -----------------------------------------------------------
  public void neueKategorie() {
    final String OHNE = "{ohne}";
    DlgMaster dlg = new DlgMaster("Neue Kategorie", getOberflaecheHaushalt());
    String[] kategorien = Kategorie.getKategorien(false);
    int size = kategorien.length;
    String[] auswahl = new String[size+1];
    auswahl[0] = OHNE;
    System.arraycopy(kategorien,0,auswahl,1,size);
    PaneKategorie pane1 = new PaneKategorie("Unter-Kategorie von:", auswahl);
    dlg.addPane(pane1);
    PaneText pane2 = new PaneText("Name der neuen (Unter-)Kategorie:", "");
    dlg.addPane(pane2);

    if(dlg.showDialog()) {
      String kategorie = (String)pane1.getWert();
      if(kategorie == OHNE)
        Kategorie.lookup((String)pane2.getWert());
      else
        Kategorie.lookup(kategorie+":"+pane2.getWert());
    }
  }

// --- alteBuchungenLoeschen ---------------------------------------------------
  public void alteBuchungenLoeschen() {
    Register register = null;
    DlgMaster dlg = new DlgMaster("Alte Buchungen lschen", getOberflaecheHaushalt());
    PaneDatum pane = new PaneDatum("Datum bis zu dem alle Buchungen gelscht werden:", new Datum());
    dlg.addPane(pane);

    if(dlg.showDialog())
      Register.entfernenAlteBuchungen((Datum)pane.getWert());
  }

// --- kategorieErsetzen -------------------------------------------------------
  public void kategorieErsetzen() {
    DlgMaster dlg = new DlgMaster("Kategorie ersetzen", getOberflaecheHaushalt());
    PaneKategorie pane1 = new PaneKategorie("Alte Kategorie:", Kategorie.getKategorien(true));
    dlg.addPane(pane1);
    PaneKategorie pane2 = new PaneKategorie("Neue Kategorie:", Kategorie.getKategorien(true));
    dlg.addPane(pane2);

    if(dlg.showDialog()) {
      Kategorie alteKategorie = Kategorie.lookup((String)pane1.getWert());
      Kategorie neueKategorie = Kategorie.lookup((String)pane2.getWert());
      Register.ersetzeKategorie(alteKategorie, neueKategorie);
    }
  }

// --- registerVereinigen ------------------------------------------------------
  public void registerVereinigen() {
    DlgMaster dlg = new DlgMaster("Register vereinigen ...", getOberflaecheHaushalt());
    PaneKategorie pane1 = new PaneKategorie(
      "Bitte whlen Sie ein Register zum Lschen aus (Quelle):",
      Register.getRegisterNamen());
    dlg.addPane(pane1);
    PaneKategorie pane2 = new PaneKategorie(
      "Bitte whlen Sie ein Register zum Einfgen aus (Ziel):",
      Register.getRegisterNamen());
    dlg.addPane(pane2);

    if(dlg.showDialog()) {
      String quelle = pane1.getWert().toString();
      String ziel = pane2.getWert().toString();
      Register.findeRegister(ziel).registerVereinigen(quelle);
      getOberflaecheHaushalt().entferneRegister(quelle);
    }
  }

// === menuExtras ==============================================================

// --- optionen ----------------------------------------------------------
  protected void initOptionen() {
    super.initOptionen();

    getDlgOptionen().addCheckBox(
      "UNTERKAT", "Unterkategorien verwenden:", unterkat);

    boolean gemerkte = Boolean.valueOf(properties.getProperty(kuerzel+".opt.gemerkte", "true")).booleanValue();
    getDlgOptionen().addCheckBox(
      "GEMERKTE", "Gemerkte Buchungen verwenden:", gemerkte);

    String maxtorte = properties.getProperty(kuerzel+".opt.maxtorte", "10");
    getDlgOptionen().addTextField(
      "MAXTORTE", "Anzahl Werte (Tortendiagramm):", maxtorte);

    String[] deltasteAuswahl = {"Entf", "Shift-Entf", "Strg-Entf", "Alt-Entf"};
    String deltaste = properties.getProperty(kuerzel+".opt.deltaste", deltasteAuswahl[0]);
    getDlgOptionen().addComboBox(
      "DELTASTE", "Taste Zellen-Lschen:", deltasteAuswahl, deltaste);

    String[] registerAuswahl = {"BOTTOM", "TOP", "LEFT", "RIGHT"};
    String reiter = properties.getProperty(kuerzel+".opt.reiter", registerAuswahl[0]);
    getDlgOptionen().addComboBox(
      "REITER", "Ort Register-Auswahl:", registerAuswahl, reiter);

    String farbschema = properties.getProperty(kuerzel+".opt.farbschema", TortenPane.schemaNamen[0]);
    getDlgOptionen().addComboBox(
      "FARBSCHEMA", "Farbschema (Tortendiagramm):", TortenPane.schemaNamen, farbschema);
  }

  protected void uebertrageOptionen() {
    super.uebertrageOptionen();

    Boolean unterkat = (Boolean)getDlgOptionen().getWert("UNTERKAT");
    properties.setProperty(kuerzel+".opt.unterkat", unterkat.toString());
    this.unterkat = unterkat.booleanValue();

    Boolean gemerkte = (Boolean)getDlgOptionen().getWert("GEMERKTE");
    properties.setProperty(kuerzel+".opt.gemerkte", gemerkte.toString());
    Register.setGemerkteBuchungenVerwenden(gemerkte.booleanValue());

    String maxtorte = (String)getDlgOptionen().getWert("MAXTORTE");
    properties.setProperty(kuerzel+".opt.maxtorte", maxtorte);

    String deltaste = (String)getDlgOptionen().getWert("DELTASTE");
    properties.setProperty(kuerzel+".opt.deltaste", deltaste);
    BuchungsEditor.setDeltaste(deltaste);

    String reiter = (String)getDlgOptionen().getWert("REITER");
    properties.setProperty(kuerzel+".opt.reiter", reiter);
    getOberflaecheHaushalt().setTabPlacement(reiter);

    String farbschema = (String)getDlgOptionen().getWert("FARBSCHEMA");
    properties.setProperty(kuerzel+".opt.farbschema", farbschema);
  }

// --- autoBuchung -------------------------------------------------------------
  public void autoBuchung() {
    DlgAutoBuchung dlg = new DlgAutoBuchung(getOberflaecheHaushalt(), autoBuchung);
    dlg.showDialog();
  }

// --- registerAnzeigen --------------------------------------------------------
  public void registerAnzeigen() {
    Register reg = Register.getRegister()[0];
    String[] text = {"Leer"};
    final MultiPageViewer dlg = new MultiPageViewer(getOberflaecheHaushalt(), reg.toString(), text, getSeitenFormat());
    String font = properties.getProperty(kuerzel+".opt.font", "SansSerif");
    int punkt = Integer.parseInt(properties.getProperty(kuerzel+".opt.punkt", "12"));
    dlg.setFont(font, punkt);
    double[] relTabs = {0.0, 15.0, 50.0, 85.0};
    dlg.setTabs(relTabs);
    JPanel pane = new JPanel();
    final IntervallComboBox intervallComboBox = new IntervallComboBox();
    final JTextField textField = new JTextField(25);
    final JComboBox registerComboBox = new JComboBox(Register.getRegister());
    textField.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Register register = (Register)registerComboBox.getSelectedItem();
        ZeitIntervall zeitIntervall = intervallComboBox.getZeitIntervall(textField.getText());
        String[] text = register.getBuchungen(zeitIntervall);
        if(DEBUG)
          System.out.println(""+text.length+" Buchungen im Zeitraum "+zeitIntervall);
        dlg.setText(text);
        dlg.setUeberschrift(register.toString());
      }
    });
    pane.add(intervallComboBox);
    pane.add(textField);
    pane.add(registerComboBox);
    dlg.setAuswahlPane(pane);
    dlg.showDialog();
  }

// --- kategorieAnzeigen -------------------------------------------------------
  public void kategorieAnzeigen() {
    Register reg = Register.getRegister()[0];
    String[] text = {"Leer"};
    final MultiPageViewer dlg = new MultiPageViewer(getOberflaecheHaushalt(), reg.toString(), text, getSeitenFormat());
    String font = properties.getProperty(kuerzel+".opt.font", "SansSerif");
    int punkt = Integer.parseInt(properties.getProperty(kuerzel+".opt.punkt", "12"));
    dlg.setFont(font, punkt);
    double[] relTabs = {0.0, 15.0, 50.0, 85.0};
    dlg.setTabs(relTabs);
    JPanel pane = new JPanel();
    final IntervallComboBox intervallComboBox = new IntervallComboBox();
    final JTextField textField = new JTextField(25);
    final JComboBox kategorieComboBox = new JComboBox(Kategorie.getKategorien(unterkat));
    textField.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String kategorie = (String)kategorieComboBox.getSelectedItem();
        ZeitIntervall zeitIntervall = intervallComboBox.getZeitIntervall(textField.getText());
        String[] text = Register.getBuchungen(zeitIntervall, kategorie, unterkat);
        if(DEBUG)
          System.out.println(""+text.length+" Buchungen im Zeitraum "+zeitIntervall);
        dlg.setText(text);
        dlg.setUeberschrift(kategorie);
      }
    });
    pane.add(intervallComboBox);
    pane.add(textField);
    pane.add(kategorieComboBox);
    dlg.setAuswahlPane(pane);
    dlg.showDialog();
  }

// === menuHilfe ===============================================================

// --- performanzTest ----------------------------------------------------------
  public void performanzTest() {
    if(!neu())
      return;
    monitor.setNote("Erzeuge Kategorien.");
    final int anzahl = 300;
    monitor.setMaximum(2*anzahl + anzahl*anzahl);
    monitor.setProgress(0);
    final SwingWorker worker = new SwingWorker() {
        public Object construct() {
          long start = new Date().getTime();
          int c = 0;
          for(int i=0; i<anzahl; i++) {
            Kategorie.lookup("Kategorie "+i);
          }
          monitor.setProgress(anzahl);
          monitor.setNote("Erzeuge Register.");
          for(int i=0; i<anzahl; i++) {
            Register.erzeugeRegister("Register "+i);
          }
          monitor.setProgress(2*anzahl);
          for(int i=0; i<anzahl; i++) {
            monitor.setNote("Erzeuge Buchungen (Reg. "+i+")");
            Register reg = Register.findeRegister("Register "+i);
            for(int j=0; j<anzahl; j++) {
              SplitBetrag splitBetrag = new SplitBetrag("Kategorie "+j, new Euro(-13.0D));
              reg.einfuegenBuchung(new StandardBuchung(new Datum(), "Buchung "+j, splitBetrag));
            }
            for(int j=0; j<anzahl; j++) {
              reg.einfuegenBuchung(new Umbuchung(new Datum(), "Umbuchung "+j, "Register "+j, new Euro(100.0D)));
            }
            if(monitor.isCanceled())
              i = anzahl-1;
            monitor.setProgress((i+3)*anzahl);
          }
          getOberflaecheHaushalt().zeigeAlleRegister();
          monitor.setProgress(2*anzahl + anzahl*anzahl);
          fehlerAusgabe.println("-I- Der Performanz-Test bentigte "+(new Date().getTime()-start)+" ms");
          geaendert(false);
          return null;
        }
    };
    worker.start();
  }

  public static void main(String[] args) {
    OberflaecheHaushalt oberflaeche = new OberflaecheHaushalt();
    Haushalt haushalt = new Haushalt(oberflaeche);
    oberflaeche.pack();
    oberflaeche.setVisible(true);
    if(DEBUG)
      System.out.println("Oberflche sichtbar.");
  }
}