package haushalt;
import haushalt.werte.*;
import app.Applikation;

import java.util.ArrayList;
import javax.swing.JTable;
import javax.swing.JOptionPane;
import javax.swing.table.*;
import java.io.*;
import java.awt.event.*;

/**
 * @author Lars H. Hahn
 * @version 1.1
 */

public class SplitBetrag
  extends AbstractTableModel
  implements Speicherbar, Cloneable, KeyListener {

  private ArrayList splitKategorie = new ArrayList();
  private ArrayList splitBetrag  = new ArrayList();

  public SplitBetrag() {
    add(Kategorie.SONSTIGES, new Euro());
  }

  public SplitBetrag(String kategorie, Euro betrag) {
    add(kategorie, betrag);
  }

  public Object clone() {
    SplitBetrag kopie = new SplitBetrag();
    kopie.splitBetrag.clear(); // initiale 0 Euro loeschen!
    for(int i=0;i<splitKategorie.size();i++) {
      kopie.splitBetrag.add(((Euro)this.splitBetrag.get(i)).clone());
    }
    // Kategorien NICHT clonen, da dann nicht in der Kategorie-Liste
    kopie.splitKategorie = (ArrayList)this.splitKategorie.clone();
    return kopie;
  }

  public boolean isSplit() {
    if(splitKategorie.size() > 1)
      return true;
    return false;
  }

  public boolean sucheText(String text) {
    for(int i=0;i<splitKategorie.size();i++)
      if(text.equals(splitKategorie.get(i).toString()))
        return true;
    return false;
  }

  public boolean ersetzeText(String alt, String neu) {
    for(int i=0;i<splitKategorie.size();i++)
      if((alt.equals(splitKategorie.get(i).toString())) && Kategorie.isKategorie(neu)) {
        splitKategorie.set(i, Kategorie.lookup(neu));
        return true;
      }
    return false;
  }

  public void add(String kategorie, Euro betrag) {
    add(Kategorie.lookup(kategorie), betrag);
  }

  private void add(Kategorie kategorie, Euro betrag) {
    splitKategorie.add(kategorie);
    splitBetrag.add(betrag);
  }

  public String toString() {
    if(splitKategorie.size() == 1)
      return ""+splitKategorie.get(0);
    return "{Split-Buchung}";
  }

  public Euro getWert() {
    Euro summe = new Euro();
    for(int i=0;i<splitKategorie.size();i++)
      summe = summe.add((Euro)splitBetrag.get(i));
    return summe;
  }

  public void setWert(Euro wert) {
    if(isSplit())
      Applikation.fehlerAusgabe.println("-E- Der Wert einer Split-Buchung kann nicht pauschal gesetzt werden!");
    else
      splitBetrag.set(0, wert);
  }

  public void summieren(ZeitIntervall intervall) {
    for(int i=0;i<splitKategorie.size();i++)
      ((Kategorie)splitKategorie.get(i)).addEuro((Euro)splitBetrag.get(i));
  }

  public void ersetzeKategorie(Kategorie alteKategorie, Kategorie neueKategorie) {
    for(int i=0; i<splitKategorie.size(); i++)
      if(alteKategorie == splitKategorie.get(i))
        splitKategorie.set(i, neueKategorie);
  }

  public Euro getKategorieWert(Kategorie kategorie, boolean unterkategorien) {
    for(int i=0; i<splitKategorie.size(); i++) {
      Kategorie aktuelleKategorie = (Kategorie)splitKategorie.get(i);
      if(!unterkategorien)
        aktuelleKategorie = Kategorie.lookup(aktuelleKategorie.getHauptkategorie());
      if(kategorie == aktuelleKategorie)
        return (Euro)splitBetrag.get(i);
    }
    return null;
  }

  /****************************************************************************
   * Die folgenden Methoden berschreiben Methoden der Basisklasse
   * AbstractTableModel. Sie werden von JTable verwendet.
   *
   */

  public int getColumnCount() {
    return 3;
  }

  public String getColumnName(int col) {
    switch(col) {
      case 0: return "Kategorie";
      case 1: return "Betrag ()";
      case 2: return "Summe";
    }
    return "Fehler";
  }

  public Class getColumnClass(int col) {
    switch(col) {
      case 0: return Kategorie.class;
      case 1: return Euro.class;
      case 2: return String.class;
    }
    return null;
  }

  public Object getValueAt(int row, int col) {
    if(row < splitBetrag.size())
      switch(col) {
        case 0: return splitKategorie.get(row);
        case 1: return splitBetrag.get(row);
        case 2: {
                  Euro summe = new Euro();
                  for(int i=0; i<=row; i++)
                    summe = summe.add((Euro)splitBetrag.get(i));
                  return summe;
                }
      }
    return "";
  }

  public boolean isCellEditable(int row, int col) {
    if(col > 1)
      return false;
    return true;
  }

  public void setValueAt(Object value, int row, int col) {
    if(row == splitBetrag.size())
      add(Kategorie.SONSTIGES, new Euro());

    switch(col) {
      case 0: splitKategorie.set(row, Kategorie.lookup((String)value)); break;
      case 1: splitBetrag.set(row, (Euro)value); break;
    }
    fireTableCellUpdated(row, col);
    fireTableDataChanged(); // Neuzeichnen der gesamten Tabelle
  }

  public int getRowCount() {
    return splitBetrag.size()+1;
  }

  /*
   * Methoden fuer Interface: Speicherbar
   */

  public void laden(DataInputStream in)
    throws IOException {
    int size = in.readInt();
    splitKategorie = new ArrayList(size);
    splitBetrag = new ArrayList(size);
    for(int i=0; i<size; i++) {
      String kategorie = in.readUTF();
      Euro betrag = new Euro();
      betrag.laden(in);
      this.add(kategorie, betrag);
    }
  }

  public void speichern(DataOutputStream out)
    throws IOException {
    out.writeInt(splitKategorie.size());
    for(int i=0; i<splitKategorie.size(); i++) {
      out.writeUTF(((Kategorie)splitKategorie.get(i)).toString());
      ((Euro)splitBetrag.get(i)).speichern(out);
    }
  }

  /*
   * Methoden des Interface 'KeyListener'
   */

  public void keyReleased(KeyEvent e) {
    JTable table = (JTable)e.getSource();
    if((e.getModifiers() == 8) && (e.getKeyCode() == 'D')) { // Alt-D
      int row = table.getSelectedRow();
      if(row == -1) {
        table.getCellEditor().cancelCellEditing();
        JOptionPane.showMessageDialog(null,
          "Keine Zeile selektiert!",
          "Alt-D: SplitBuchung",
          JOptionPane.WARNING_MESSAGE);
      }
      else if(row < splitKategorie.size()){
        table.getCellEditor().cancelCellEditing();
        splitKategorie.remove(row);
        splitBetrag.remove(row);
        fireTableDataChanged(); // Neuzeichnen der Tabelle
      }
    }
  }

  public void keyPressed(KeyEvent e){};

  public void keyTyped(KeyEvent e){};
}