package haushalt.werte;

import java.io.*;
import java.text.DateFormat;
import java.util.*;

/**
 * Teil des Paketes 'werte' des Projektes jHaushalt.
 *
 * @author Lars H. Hahn
 * @version 1.1
 */

public class Datum
  implements Comparable, Speicherbar {

  static final DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, Locale.GERMAN);
  private Date wert;

  public Datum() {
    wert = new Date();
  }

  public Datum(int tag, int monat, int jahr) {
    GregorianCalendar cal = new GregorianCalendar(jahr, monat-1, tag);
    wert = cal.getTime();
  }

  public Datum(String wert) {
    String heute = df.format(new Date());
    if(wert.length() < 8)
      wert = wert + heute.substring(wert.length());
    try {
      this.wert = df.parse(wert);
    }
    catch(java.text.ParseException e) {
      this.wert = new Date();
    }
  }

  public long getTime() {
    return wert.getTime();
  }

  public Datum addTage(int tage) {
    Datum datum = new Datum();
    datum.wert = new Date(getTime()+tage*86400000L);
    return datum;
  }

  public int getTag() {
    GregorianCalendar gc = new GregorianCalendar();
    gc.setTime(wert);
    return gc.get(Calendar.DAY_OF_MONTH);
  }

  public int getMonat() {
    GregorianCalendar gc = new GregorianCalendar();
    gc.setTime(wert);
    return gc.get(Calendar.MONTH)+1;
  }

  public int getJahr() {
    GregorianCalendar gc = new GregorianCalendar();
    gc.setTime(wert);
    return gc.get(Calendar.YEAR);
  }

  public String toString() {
    return df.format(wert);
  }

  public boolean istImIntervall(ZeitIntervall intervall) {
    return ((compareTo(intervall.getStartDatum()) >= 0) &&
            (compareTo(intervall.getEndDatum()) < 0));
  }

  /*
   * Methoden fuer Interface: Comparable
   */

  public int compareTo(Object obj) {
    Date date = ((Datum)obj).wert;
    return wert.compareTo(date);
  }

  /*
   * Methoden fuer Interface: Speicherbar
   */

  public void laden(DataInputStream in)
    throws IOException {
    try {
      wert = df.parse(in.readUTF());
    }
    catch(java.text.ParseException e) {
      wert = new Date();
      throw new IOException();
    }
  }

  public void speichern(DataOutputStream out)
    throws IOException {
    out.writeUTF(toString());
  }

}