package haushalt.werte;

import java.io.*;
import java.text.NumberFormat;
import java.util.Locale;

/**
 * Teil des Paketes 'werte' des Projektes jHaushalt.
 *
 * @author Lars H. Hahn
 * @version 1.1
 */

public class Euro
  implements Cloneable, Comparable, Speicherbar {

  public static final Euro NULL_EURO = new Euro();
  private static final NumberFormat nf = NumberFormat.getInstance(Locale.GERMAN);
  private long wert = 0L;

  public Euro() {
  }

  public Euro(double wert) {
    setWert(wert);
  }

  public Euro(String wert) {
    if(wert != "") {
      try {
        if(wert.endsWith(""))
          setWert(nf.parse(wert.substring(0, wert.length()-1)).doubleValue());
        else
          setWert(nf.parse(wert).doubleValue());
      }
      catch(java.text.ParseException e) {
        // Kein Problem: Der Wert ist 0 Euro!
      }
    }
  }

  public Object clone() {
    Euro kopie = new Euro();
    kopie.wert = this.wert;
    return kopie;
  }

  private void setWert(double wert) {
    double help = wert * 100.0D;
    if(help < 0)
      this.wert = (long) (help - 0.5D);
    else
      this.wert = (long) (help + 0.5D);
  }

  public void umrechnenVonDM() { // Wenn der Wert in DM vorliegt -> umrechnen in EURO
    if(wert < 0)
      wert = (long)(wert / 1.95583D - 0.5D);
    else
      wert = (long)(wert / 1.95583D + 0.5D);
  }

  public String toString() {
    nf.setMinimumFractionDigits(2);
    nf.setMaximumFractionDigits(2);
    return nf.format(wert / 100.0D) + " ";
  }

  public double toDouble() {
    return wert / 100.0;
  }

  public boolean equals(Object obj) {
    if(obj == null)
      return false;
    if(obj.getClass() != this.getClass())
      return false;
    return ((Euro)obj).wert == wert;
  }

  public Euro add(Euro euro) {
    Euro ergebnis = new Euro();
    ergebnis.wert = this.wert + euro.wert;
    return ergebnis;
  }

  public Euro sub(Euro euro) {
    Euro ergebnis = new Euro();
    ergebnis.wert = this.wert - euro.wert;
    return ergebnis;
  }

  public Euro durch(int zahl) {
    Euro ergebnis = new Euro();
    ergebnis.wert = this.wert / zahl;
    return ergebnis;
  }

  /*
   * Methoden fuer Interface: Comparable
   */

  public int compareTo(Object obj) {
    if(wert < ((Euro)obj).wert)
      return -1;
    if(wert > ((Euro)obj).wert)
      return 1;
    return 0;
  }

  /*
   * Methoden fuer Interface: Speicherbar
   */

  public void laden(DataInputStream in)
    throws IOException {
    wert = in.readLong();
  }

  public void speichern(DataOutputStream out)
    throws IOException {
    out.writeLong(wert);
  }

}